-- Title	: 40-21 Converter
-- Rev 		: 3.1.0
-- conv_out.vhd


library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

entity CONV_OUT is
  port
    (
    BCLK            :in       std_logic;
    BCLK2X          :in       std_logic;
    PORT1           :in       std_logic;
    PORT2           :in       std_logic;
    DRST            :in       std_logic;
    AUX0            :in       std_logic;
    AUX1            :in       std_logic;
    DATAIN          :in       std_logic_vector(31 downto 0);
    PAR             :in       std_logic_vector(3 downto 0);
    BRSTn           :in       std_logic;

    LINK1           :out      std_logic;
    LINK2           :out      std_logic;
    DATAOUT         :out      std_logic_vector(19 downto 0);

    O_OPTIN         :in       std_logic_vector(1 downto 0);
    O_OPTOUT        :out      std_logic_vector(1 downto 0)
    );
end CONV_OUT;

architecture MAINBODY of CONV_OUT is

        component  genpar32
        port(
            din : in std_logic_vector(31 downto 0);
            pout : out std_logic_vector(3 downto 0)
            );
    end component;
    component jpburst 
    PORT(
        rst		: IN	STD_LOGIC;
        CLK		: IN	STD_LOGIC;
        phase		: IN	STD_LOGIC;
        aux1		: IN	STD_LOGIC;
        we1		: IN	STD_LOGIC;
        we2		: IN	STD_LOGIC;
        bstmax        : in   std_logic_vector(5 downto 0);
        spcmax        : in   std_logic_vector(5 downto 0);
        version         : in   std_logic;
        jpmask          : out   std_logic
	);
   end component;

    component regwrite
    PORT(
        rst		: IN	STD_LOGIC;
        CLK		: IN	STD_LOGIC;
        phase		: IN	STD_LOGIC;
        aux1		: IN	STD_LOGIC;
        we		: IN	STD_LOGIC;
        din		: IN	std_logic_vector(15 downto 0);
        bstcount   : out   std_logic_vector(5 downto 0);
        spccount   : out   std_logic_vector(5 downto 0);
        version     : out   std_logic
	);
   END component;



signal  INTDATA	    :std_logic_vector(31 downto 0);
signal  INTDATAF    :std_logic_vector(31 downto 0);
signal  PHASE       :std_logic;
signal  INTDRST     :std_logic;
signal  INTPORT1    :std_logic;
signal  INTPORT2    :std_logic;
signal  INTPAR      :std_logic_vector(3 downto 0);
signal  pout        :std_logic_vector(3 downto 0);
signal  INTAUX0     :std_logic;
signal  INTAUX1     :std_logic;

signal  bstcount    :std_logic_vector(5 downto 0);
signal  spccount    :std_logic_vector(5 downto 0);
signal  version     :std_logic;

signal port1n       :std_logic;
signal port2n       :std_logic;
signal jpmask       :std_logic;
begin

----- generate phase signal from BCLK ---------------------------------
  process (BCLK2X) begin
    if (BCLK2X'event and BCLK2X = '1') then
      PHASE <= BCLK;
    end if;
  end process;

----- generate DRSTD from DRST    -------------------------------------
  process (BCLK2X) begin
    if (BCLK2X'event and BCLK2X = '1') then
      if (PHASE = '1') then
        INTDRST  <= DRST;
        INTPORT1 <= PORT1;
        INTPORT2 <= PORT2;
        INTAUX0  <= AUX0;
        INTAUX1  <= AUX1;
      end if;
    end if;
  end process;

----- store parity bit from mainc(FPGA) -------------------------------
--  process (BCLK2X,BRSTn) begin
--    if (BRSTn = '0') then
--      INTPAR <= "0000";
--    elsif (BCLK2X'event and BCLK2X = '1') then
--      if ((PHASE = '1') and ((PORT1 = '1') or (PORT2 = '1'))) then
--        INTPAR <= PAR;
--      end if;
--    end if;
--  end process;

----- store INPUT data from 9080 --------------------------------------
  process (BCLK2X)
  begin
      if (BCLK2X'event and BCLK2X = '1') then
          if (  PHASE = '1') then
              INTDATAF <= DATAIN;
          end if;
      end if;
  end process;

  u_pout: genpar32 port map(intdataf,pout);
  process (BCLK2X)
  begin
      if (BCLK2X'event and BCLK2X = '1') then
          if (PHASE = '1')  then
              INTDATA <= INTDATAF;
              intpar  <= pout;
          end if;
      end if;
  end process;


----- select outputdata & AUX -------------------------------------------
  process (BCLK2X) begin
    if (BCLK2X'event and BCLK2X = '1') then
      if (PHASE = '0') then
        DATAOUT(19)          <= INTAUX0;
        DATAOUT(18)          <= '1';                   --phase
        DATAOUT(17)          <= INTPAR(3);
        DATAOUT(16 downto 9) <= INTDATA(31 downto 24);
        DATAOUT(8)           <= INTPAR(2);
        DATAOUT(7 downto 0)  <= INTDATA(23 downto 16);
      else
        DATAOUT(19)          <= INTAUX1;
        DATAOUT(18)          <= '0';                   --phase
        DATAOUT(17)          <= INTPAR(1);
        DATAOUT(16 downto 9) <= INTDATA(15 downto 8);
        DATAOUT(8)           <= INTPAR(0);
        DATAOUT(7 downto 0)  <= INTDATA(7 downto 0);
      end if;
    end if;
  end process;

----- output DS & RST ---------------------------------------------
  process (BCLK2X,BRSTn) begin
    if (BRSTn = '0') then
      LINK1 <= '1';
      LINK2 <= '1';
    elsif (BCLK2X'event and BCLK2X = '1') then
      if (PHASE = '0') then
        LINK1 <= not(INTPORT1 and jpmask);
        LINK2 <= not(INTPORT2 and jpmask);
      else
        LINK1 <= INTDRST;
        LINK2 <= INTDRST;
      end if;
    end if;
  end process;

----- option signal for future expansion---------------------------
  O_OPTOUT <= "11";


----- JPDMA spacing control additional logics --------------------
    port1n <= not INTPORT1;	
    port2n <= not INTPORT2;	
    u_regwrite: regwrite port map(
        rst=>INTDRST,
        CLK=>BCLK2X,
        phase=>PHASE,
        aux1=>INTAUX1,
        we=>port1n,
        din=>INTDATA(15 downto 0),
        bstcount=>bstcount,
        spccount=>spccount,
        version=>version);

    u_jpburst: jpburst port map(
        rst=>INTDRST,
        CLK=>BCLK2X,
        phase=>PHASE,
        aux1=>INTAUX1,
        we1=>port1n,
        we2=>port2n,
        bstmax=>bstcount,
        spcmax=>spccount,
        version=>version,
        jpmask=>jpmask);

end MAINBODY;


