-- convert3618.vhd
-- convert 18bit data to 36 bit data
-- Version 0.0 1999/8/27 J. Makino
-- Version 0.1 2000/2/16 To be consistent with Koga-kun board I/O definition
-- Version 0.2 2000/2/25 Apparent timing problem with DS/RST fixed (hopefully)


LIBRARY ieee;
USE ieee.std_logic_1164.all;
USE ieee.std_logic_unsigned.all;


ENTITY convert3618 IS

	PORT(
		CLK		: IN	STD_LOGIC;
		CLK2		: IN	STD_LOGIC;
		din		: IN	std_logic_vector(35 downto 0);
		dsin		: IN 	std_logic;
                rstin         : IN    std_logic;
                auxin             : IN    std_logic_vector(1 downto 0);
		dout		: OUT 	std_logic_vector(20 downto 0)
	);
	
END convert3618;

ARCHITECTURE a OF convert3618 IS


signal inreg : std_logic_vector(39 downto 0);
signal inreg2 : std_logic_vector(39 downto 0);
signal inreg3 : std_logic_vector(39 downto 0);
signal outreg : std_logic_vector(20 downto 0);
signal phase : std_logic;
BEGIN

    process(clk2)
    begin
        if(clk2'event and clk2='1') then
--            The following is the data location definition as specified in
--            Makino's document
--            inreg(17 downto 0) <= din(17 downto 0);
--            inreg(18)<= rstin;
--            inreg(19)<= auxin(0);
--            inreg(37 downto 20) <= din(35 downto 18);
--            inreg(38)<= dsin;
--            inreg(39)<= auxin(1);
--            The following is the data location definition as implemented 
--            in Koga-kun board
--            The difference is that this is (8+1)x4, while the specification
--            is 32+4
            
            inreg(7 downto 0) <= din(7 downto 0);
            inreg(16 downto 9) <= din(15 downto 8);
            inreg(27 downto 20) <= din(23 downto 16);
            inreg(36 downto 29) <= din(31 downto 24);
            inreg(8) <= din(32);
            inreg(17) <= din(33);
            inreg(28) <= din(34);
            inreg(37) <= din(35);
            
            inreg(18)<= rstin;
            inreg(19)<= auxin(0);
            inreg(38)<= dsin;
            inreg(39)<= auxin(1);
        end if;
    end process;
    process(clk)
    begin
        if(clk'event and clk='1') then
            inreg3 <= inreg;
            if clk2 = '1' then
                phase <= '0';
                inreg2 <= inreg3;
                outreg(17 downto 0) <= inreg2(17 downto 0);
                outreg(18) <= '0';
-- For some unknown reason, the next two lines used to use inreg instead of inreg2
-- corrected in version 0.2, 2000/2/25
                outreg(19) <= inreg2(18);
                outreg(20) <= inreg2(19);
            else
                phase <= '1';
                outreg(17 downto 0) <= inreg2(37 downto 20);
                outreg(18) <= '1';
                outreg(19) <= inreg2(38);
                outreg(20) <= inreg2(39);
            end if;
        end if;
    end process;

    dout <= outreg;
END a;


