-- fofpgawrite.vhd
-- snooping the IP link and write the serial programming port of fofpga
--
-- Version 0.0 1999/9/13 J. Makino
-- Version 0.1 1999/11/2 J. Makino
-- Main changs: nconfigout changed to bus line (5 lines)



LIBRARY ieee;
USE ieee.std_logic_1164.all;
USE ieee.std_logic_unsigned.all;


ENTITY fofpga_write IS
    PORT(
        rst		: IN	STD_LOGIC;
        CLK		: IN	STD_LOGIC;
        we		: IN	STD_LOGIC;
        din		: IN	std_logic_vector(15 downto 0);
        nconfigout      : out   std_logic_vector(4 downto 0);
        data0out        : out   std_logic;
        dclkout         : out   std_logic
	);
END fofpga_write;

ARCHITECTURE a OF fofpga_write IS

    signal header_state : std_logic;
    signal dcount : std_logic_vector(10 downto 0);
    signal endcount : std_logic_vector(10 downto 0);
    signal dinreg : std_logic_vector(15 downto 0);
    signal address : std_logic_vector(15 downto 0);
    signal wereg  : std_logic;

BEGIN

-- input data registers 
    process(clk)
    begin
        if(clk'event and clk='1') then
            dinreg <= din;
            wereg <=we;
        end if;
    end process;

-- header state
    process(clk)
    begin
        if(clk'event and clk='1') then
            if (rst = '0') then
                header_state <= '0';
            elsif (wereg = '0') then
                if (header_state = '0' and dcount = 0) then
                    header_state <= '1';
                else
                    header_state <= '0';
                end if;
            end if;
        end if;
    end process;
    
-- data counter
    process(clk)
    begin
        if(clk'event and clk='1') then
            if (rst = '0') then
                dcount <= "00000000000";
            elsif (wereg = '0') then
                if (header_state = '1') then
                    dcount <= dinreg(10 downto 0);
                elsif (dcount /= 0) then
                    dcount <= dcount - 1;
                end if;
            end if;
        end if;
    end process;

-- address counter 
    process(clk)
    begin
        if(clk'event and clk='1') then
            if (wereg = '0') then
                if( header_state = '0' and dcount = 0 ) then
                    address<= dinreg;
                elsif( header_state = '0' ) then
                    address <= address + 1;
                end if;
            end if;
        end if;
    end process;

-- load N for the address of N
    process(clk)
    begin
        if(clk'event and clk='1') then
-- use the first word of IPREG reserved region  00 0100 0010 0000
--                                
            if (wereg = '0' and address = 16#0420# and dcount /=0 ) then
                nconfigout <= dinreg(6 downto 2);
                data0out <= dinreg(1);
                dclkout <= dinreg(0);
            end if;
        end if;
    end process;
END a;











