-- Rev          : 3.1.0

library ieee;
use ieee.std_logic_1164.all;

entity CONV_IN is
  port(
      DCLKn         :in       std_logic;
      DCLK_2        :in       std_logic;
      DATAIN        :in       std_logic_vector(20 downto 0);

      DATAOUT       :out      std_logic_vector(39 downto 0);
      COUNT         :out      std_logic;

      OPTIONIN      :in       std_logic_vector(1 downto 0);
      OPTIONOUT     :out      std_logic_vector(1 downto 0)
      );
end CONV_IN;


architecture MAINBODY of CONV_IN is

signal DATAH        :std_logic_vector(20 downto 0);
signal INTDATA      :std_logic_vector(20 downto 0);
signal PHASE        :std_logic;
signal INTDATAS     :std_logic_vector(41 downto 0);

begin

-----First f/f to latch input data----------------------------------
  process (DCLKn) begin
    if (DCLKn'event and DCLKn = '1') then
      INTDATA <= DATAIN;
     end if;
  end process;

-----produce PHASE signal from DCLK2/-------------------------------
  process (DCLKn) begin
    if (DCLKn'event and DCLKn = '1') then
      PHASE <= DCLK_2;
    end if;
  end process;

-----adopt data to higher bits or lower bits depend on PHASE signal-
  process (DCLKn) begin
    if (DCLKn'event and DCLKn = '1') then
      if (INTDATA(18) = '1') then
        DATAH <= INTDATA;
      end if;
    end if;
  end process;

-----transfer data to 2nd F/F when (PHASE=1)---------------------------
  process (DCLKn) begin
    if (DCLKn'event and DCLKn = '1') then       
      if (INTDATA(18) = '0') then
        INTDATAS(20 downto 0)  <= INTDATA;
        INTDATAS(41 downto 21) <= DATAH;
      end if;
    end if;
  end process;


-----output data synchronised to DCLK_2)--------------------------
  process (DCLKn) begin
    if (DCLKn'event and DCLKn = '1') then
      if (PHASE = '1') then
        DATAOUT(18 downto 0)  <= INTDATAS(18 downto 0);
        DATAOUT(19)           <= INTDATAS(20);
        DATAOUT(38 downto 20) <= INTDATAS(39 downto 21);
        DATAOUT(39)           <= INTDATAS(41);
--        COUNT                 <= not (INTDATAS(40));   --DS active high
        COUNT                 <= INTDATAS(40);     --DS active low
      end if;
    end if;
  end process;

-----option signal for future expansion----------------------------

      OPTIONOUT <= "11";


end MAINBODY;






