-- Title	: 40-21 Converter
-- Rev 		: 3.1.0
-- conv_out_dsp.vhd
-- This version puts DS of LINK2 (signal is actually link2)
-- as additional parity input for the lowest and second lowest
-- parities

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

entity CONV_OUT_DSP is
  port
    (
    BCLK            :in       std_logic;
    BCLK2X          :in       std_logic;
    PORT1           :in       std_logic;
    PORT2           :in       std_logic;
    DRST            :in       std_logic;
    AUX0            :in       std_logic;
    AUX1            :in       std_logic;
    DATAIN          :in       std_logic_vector(31 downto 0);
    PAR             :in       std_logic_vector(3 downto 0);
    BRSTn           :in       std_logic;

    LINK1           :out      std_logic;
    LINK2           :out      std_logic;
    DATAOUT         :out      std_logic_vector(19 downto 0);

    O_OPTIN         :in       std_logic_vector(1 downto 0);
    O_OPTOUT        :out      std_logic_vector(1 downto 0)
    );
end CONV_OUT_DSP;

architecture MAINBODY of CONV_OUT_DSP is

        component  genpar32
        port(
            din : in std_logic_vector(31 downto 0);
            pout : out std_logic_vector(3 downto 0)
            );
    end component;
    

signal  INTDATA	    :std_logic_vector(31 downto 0);
signal  INTDATAF    :std_logic_vector(31 downto 0);
signal  PHASE       :std_logic;
signal  INTDRST     :std_logic;
signal  INTPORT1    :std_logic;
signal  INTPORT2    :std_logic;
signal  INTPAR      :std_logic_vector(3 downto 0);
signal  pout        :std_logic_vector(3 downto 0);
signal  pout_ds     :std_logic_vector(3 downto 0);
signal  INTAUX0     :std_logic;
signal  INTAUX1     :std_logic;

begin

----- generate phase signal from BCLK ---------------------------------
  process (BCLK2X) begin
    if (BCLK2X'event and BCLK2X = '1') then
      PHASE <= BCLK;
    end if;
  end process;

----- generate DRSTD from DRST    -------------------------------------
  process (BCLK2X) begin
    if (BCLK2X'event and BCLK2X = '1') then
      if (PHASE = '1') then
        INTDRST  <= DRST;
        INTPORT1 <= PORT1;
        INTPORT2 <= PORT2;
        INTAUX0  <= AUX0;
        INTAUX1  <= AUX1;
      end if;
    end if;
  end process;

----- store parity bit from mainc(FPGA) -------------------------------
--  process (BCLK2X,BRSTn) begin
--    if (BRSTn = '0') then
--      INTPAR <= "0000";
--    elsif (BCLK2X'event and BCLK2X = '1') then
--      if ((PHASE = '1') and ((PORT1 = '1') or (PORT2 = '1'))) then
--        INTPAR <= PAR;
--      end if;
--    end if;
--  end process;

----- store INPUT data from 9080 --------------------------------------
  process (BCLK2X)
  begin
      if (BCLK2X'event and BCLK2X = '1') then
          if (  PHASE = '1') then
              INTDATAF <= DATAIN;
          end if;
      end if;
  end process;

  u_pout: genpar32 port map(intdataf,pout);
  pout_ds(3 downto 2) <= pout(3 downto 2);
  pout_ds(1) <= pout(1) xor port2;
  pout_ds(0) <= pout(0) xor port2;

  process (BCLK2X)
  begin
      if (BCLK2X'event and BCLK2X = '1') then
          if (PHASE = '1')  then
              INTDATA <= INTDATAF;
              intpar  <= pout_ds;
          end if;
      end if;
  end process;


----- select outputdata & AUX -------------------------------------------
  process (BCLK2X) begin
    if (BCLK2X'event and BCLK2X = '1') then
      if (PHASE = '0') then
        DATAOUT(19)          <= INTAUX0;
        DATAOUT(18)          <= '1';                   --phase
        DATAOUT(17)          <= INTPAR(3);
        DATAOUT(16 downto 9) <= INTDATA(31 downto 24);
        DATAOUT(8)           <= INTPAR(2);
        DATAOUT(7 downto 0)  <= INTDATA(23 downto 16);
      else
        DATAOUT(19)          <= INTAUX1;
        DATAOUT(18)          <= '0';                   --phase
        DATAOUT(17)          <= INTPAR(1);
        DATAOUT(16 downto 9) <= INTDATA(15 downto 8);
        DATAOUT(8)           <= INTPAR(0);
        DATAOUT(7 downto 0)  <= INTDATA(7 downto 0);
      end if;
    end if;
  end process;

----- output DS & RST ---------------------------------------------
  process (BCLK2X,BRSTn) begin
    if (BRSTn = '0') then
      LINK1 <= '1';
      LINK2 <= '1';
    elsif (BCLK2X'event and BCLK2X = '1') then
      if (PHASE = '0') then
        LINK1 <= not(INTPORT1);
        LINK2 <= not(INTPORT2);
      else
        LINK1 <= INTDRST;
        LINK2 <= INTDRST;
      end if;
    end if;
  end process;

----- option signal for future expansion---------------------------
  O_OPTOUT <= "11";

end MAINBODY;

